@@ -7,16 +7,16 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id: NYTProf.pm 780 2009-06-23 20:33:57Z tim.bunce $
+## $Id: NYTProf.pm 998 2009-12-24 14:40:14Z tim.bunce $
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '2.10';
+our $VERSION = '3.00';
 
 package    # hide the package from the PAUSE indexer
     DB;
 
-# Enable specific perl debugger flags.
+# Enable specific perl debugger flags (others may be set later).
 # Set the flags that influence compilation ASAP so we get full details
 # (sub line ranges etc) of modules loaded as a side effect of loading
 # Devel::NYTProf::Core (ie XSLoader, strict, Exporter etc.)
@@ -25,20 +25,18 @@ $^P = 0x010     # record line range of sub definition
     | 0x100     # informative "file" names for evals
     | 0x200;    # informative names for anonymous subroutines
 
-# XXX hack, need better option handling
+require Devel::NYTProf::Core;    # loads XS and sets options
+
+# XXX hack, need better option handling e.g., add DB::get_option('use_db_sub')
 my $use_db_sub = ($ENV{NYTPROF} && $ENV{NYTPROF} =~ m/\buse_db_sub=1\b/);
-
-$^P |= 0x002    # line-by-line profiling via DB::DB (if $DB::single true)
-    |  0x020    # start (after BEGINs) with single-step on
-    if $use_db_sub;
-
-require Devel::NYTProf::Core;    # loads XS
-
-if ($use_db_sub) {               # install DB::DB sub
+if ($use_db_sub) {                     # install DB::DB sub
     *DB = ($] < 5.008008)
         ? sub { goto &DB_profiler }    # workaround bug in old perl versions (slow)
         : \&DB_profiler;
 }
+sub sub { die "DB::sub" }              # needed for perl <5.8.7 (<perl@24265)
+
+sub CLONE { DB::disable_profiler }
 
 init_profiler();                       # provides true return value for module
 
@@ -58,7 +56,7 @@ Devel::NYTProf - Powerful feature-rich perl source code profiler
   # convert database into a set of html files, e.g., ./nytprof/index.html
   nytprofhtml
 
-  # or into comma seperated files, e.g., ./nytprof/*.csv
+  # or into comma separated files, e.g., ./nytprof/*.csv
   nytprofcsv
 
 =head1 DESCRIPTION
@@ -77,6 +75,10 @@ Performs per-subroutine statement profiling for overview
 
 =item *
 
+Performs per-opcode profiling for slow perl builtins
+
+=item *
+
 Performs per-block statement profiling (the first profiler to do so)
 
 =item *
@@ -188,7 +190,7 @@ leaving it. It then increments a call count and accumulates the duration.
 For each subroutine called, separate counts and durations are stored I<for each
 location that called the subroutine>.
 
-Subroutine entry is detected by intercepting the entersub opcode. Subroutine
+Subroutine entry is detected by intercepting the C<entersub> opcode. Subroutine
 exit is detected via perl's internal save stack. The result is both extremely
 fast and very robust.
 
@@ -200,10 +202,19 @@ the inclusive time is only measured for the outer-most call.
 The inclusive times of recursive calls are still measured and are accumulated
 separately. Also the 'maximum recursion depth' per calling location is recorded.
 
+=head3 Goto &Subroutine
+
+Perl implements a C<goto &destination> as a C<return> followed by a call to
+C<&destination>, so that's how it will appear in the report.
+
+The C<goto> will be shown with a very short time because it's effectively just
+a C<return>. The C<&destination> sub will show a call I<not> from the location
+of the C<goto> but from the location of the call to the sub that performed the C<goto>.
+
 =head2 Application Profiling
 
 NYTProf records extra information in the data file to capture details that may
-be useful when analysing the performance. It also records the filename and line
+be useful when analyzing the performance. It also records the filename and line
 ranges of all the subroutines.
 
 NYTProf can profile applications that fork, and does so with no loss of
@@ -261,7 +272,7 @@ example:
 
   export NYTPROF=trace=2:start=init:file=/tmp/nytprof.out
 
-Any colon or equal characters in a value can be escaped by preceeding them with
+Any colon or equal characters in a value can be escaped by preceding them with
 a backslash.
 
 =head2 addpid=1
@@ -272,15 +283,21 @@ This avoids concurrent, or consecutive, processes from overwriting the same file
 
 =head2 trace=N
 
-Set trace level to N. 0 is off (the default). Higher values cause more detailed trace output.
+Set trace level to N. 0 is off (the default). Higher values cause more detailed
+trace output. Trace output is written to STDERR or wherever the L</log=F>
+option has specified.
+
+=head2 log=F
+
+Specify the name of the file that L</trace=N> output should be written to.
 
 =head2 start=...
 
 Specify at which phase of program execution the profiler should be enabled:
 
   start=begin - start immediately (the default)
-  start=init  - start at begining of INIT phase (after compilation)
-  start=end   - start at begining of END phase
+  start=init  - start at beginning of INIT phase (after compilation)
+  start=end   - start at beginning of END phase
   start=no    - don't automatically start
 
 The start=no option is handy if you want to explicitly control profiling
@@ -307,7 +324,7 @@ may be rewritten as
   1    return if (...)
 
 so the profile won't show a statement count for line 2 in your source code
-because the C<return> was merged into the C<if> statement on the preceeding line.
+because the C<return> was merged into the C<if> statement on the preceding line.
 
 Using the C<optimize=0> option disables the optimizer so you'll get lower
 overall performance but more accurately assigned statement counts.
@@ -323,7 +340,7 @@ Set to 0 to disable the collection of subroutine caller and timing details.
 
 Set to 0 to disable the determination of block and subroutine location per statement.
 This makes the profiler about 50% faster (as of July 2008) and produces smaller
-output files, but you loose some valuable information. The extra cost is likely
+output files, but you lose some valuable information. The extra cost is likely
 to be reduced in later versions anyway, as little optimization has been done on
 that part of the code.
 
@@ -338,9 +355,10 @@ profile data file.
 
 =head2 leave=0
 
-Set to 0 to disable the extra work done to allocate times accurately when
+Set to 0 to disable the extra work done by the statement profiler
+to allocate times accurately when
 returning into the middle of statement. For example leaving a subroutine
-and returning into the middle of statement, or re-evaluting a loop condition.
+and returning into the middle of statement, or re-evaluating a loop condition.
 
 This feature also ensures that in embedded environments, such as mod_perl,
 the last statement executed doesn't accumulate the time spent 'outside perl'.
@@ -348,6 +366,12 @@ the last statement executed doesn't accumulate the time spent 'outside perl'.
 NYTProf is the only line-level profiler to measure these times correctly.
 The profiler is fast enough that you shouldn't need to disable this feature.
 
+=head2 findcaller=1
+
+Force NYTProf to recalculate the name of the caller of the each sub instead of
+'inheriting' the name calculated when the caller was entered. (Rarely needed,
+but might be useful in some odd cases.)
+
 =head2 use_db_sub=1
 
 Set to 1 to enable use of the traditional DB::DB() subroutine to perform
@@ -374,12 +398,35 @@ If you're using perl 5.10.0 or 5.8.8 (or earlier) then you need to also enable
 the C<use_db_sub=1> option otherwise perl doesn't make the source code
 available to NYTProf. Perl 5.8.9 and 5.10.1+ don't require that.
 
+=head2 slowops=N
+
+Profile perl opcodes that can be slow. These include opcodes that make system
+calls, such as C<print>, C<read>, C<sysread>, C<socket> etc., plus regular
+expression opcodes like C<subst> and C<match>.
+
+If C<N> is 0 then slowops profiling is disabled.
+
+If C<N> is 1 then all the builtins are treated as being defined in the C<CORE>
+package. So times for C<print> calls from anywhere in your code are merged and
+accounted for as calls to an xsub called C<CORE::print>.
+
+If C<N> is 2 (the default) then builtins are treated as being defined in the
+package that calls them. So calls to C<print> from package C<Foo> are treated
+as calls to an xsub called C<Foo::CORE:print>. Note the single colon after CORE.
+
+The opcodes are currently profiled using their internal names, so C<printf> is C<prtf>
+and the C<-x> file test is C<fteexec>. This may change in future.
+
+Opcodes that call subroutines, perhaps by triggering a FETCH from a tied
+variable, currently appear in the call tree as the call of the sub. This is
+likely to change in future.
+
 =head2 usecputime=1
 
 Measure user CPU + system CPU time instead of the real elapsed 'wall clock'
 time (which is the default).
 
-Measuring CPU time has the advantage of making the measurements independant of
+Measuring CPU time has the advantage of making the measurements independent of
 time spent blocked waiting for the cpu or network i/o etc. But it also has the
 severe disadvantage of having typically I<far> less accurate timings.
 
@@ -415,6 +462,66 @@ If the clock you select isn't available then CLOCK_REALTIME is used.
 
 See L</CLOCKS> for more information.
 
+=head2 sigexit=1
+
+When perl exits normally it runs any code defined in C<END> blocks.
+NYTProf defines an END block that finishes profiling and writes out the final
+profile data.
+
+If the process ends due to a signal then END blocks are not executed.
+The C<sigexit> option tells NYTProf to catch some signals (e.g. INT, HUP, PIPE,
+SEGV, BUS) and ensure a usable by executing:
+
+    DB::finish_profile();
+    exit 1;
+
+You can also specify which signals to catch in this way by listing them,
+seperated by commas, as the value of the option (case is not significant):
+
+    sigexit=int,hup
+
+=head2 forkdepth=N
+
+When a perl process that is being profiled executes a fork() the child process
+is also profiled. The forkdepth option can be used to control this. If
+forkdepth is zero then profiling will be disabled in the child process.
+
+If forkdepth is greater than zero then profiling will be enabled in the child
+process and the forkdepth value in that process is decremented by one.
+
+If forkdepth is -1 (the default) then there's no limit on the number of
+generations of children that are profiled.
+
+=head2 nameevals=0
+
+The 'file name' of a string eval is normally a string like "C<(eval N)>", where
+C<N> is a sequence number. By default NYTProf asks perl to give evals more
+informative names like "C<(eval N)[file:line]>", where C<file> and C<line> are
+the file and line number where the string C<eval> was executed.
+
+The C<nameevals=0> option can be used to disable the more informative names and
+return to the default behaviour. This may be need in rare cases where the
+application code is sensitive to the name given to a C<eval>. (The most common
+case in when running test suites undef NYTProf.)
+
+The downside is that the NYTProf reporting tools are less useful and may get
+confused if this option is used.
+
+=head2 nameanonsubs=0
+
+The name of a anonymous subroutine is normally "C<__ANON__>".  By default
+NYTProf asks perl to give anonymous subroutines more informative names like
+"C<__ANON__[file:line]>", where C<file> and C<line> are the file and line
+number where the anonymous subroutine was defined.
+
+The C<nameanonsubs=0> option can be used to disable the more informative names
+and return to the default behaviour. This may be need in rare cases where the
+application code is sensitive to the name given to a anonymous subroutines.
+(The most common case in when running test suites undef NYTProf.)
+
+The downside is that the NYTProf reporting tools are less useful and may get
+confused if this option is used.
+
 =head1 RUN-TIME CONTROL OF PROFILING
 
 You can profile only parts of an application by calling DB::disable_profile()
@@ -449,6 +556,46 @@ Calls DB::disable_profile(), then completes the profile data file by writing
 subroutine profile data, and then closes the file. The in memory subroutine
 profile data is then discarded.
 
+=head1 DATA COLLECTION AND INTERPRETATION
+
+NYTProf tries very hard to gather accurate information.  The nature of the
+internals of perl mean that, in some cases, the information that's gathered is
+accurate but surprising. In some cases it can appear to be misleading.
+(Of course, in some cases it may actually be plain wrong. Caveat lector.)
+
+=head2 Perl 5.10.1+ (or else 5.8.9+) are Recommended
+
+These versions of perl yield much more detailed information about calls to
+BEGIN, CHECK, INIT, and END blocks, the code handling tied or overloaded
+variables, and callbacks from XS code.
+
+Perl 5.12 will hopefully also fix an inaccuracy in the timing of the last
+statement and the condition clause of some kinds of loops:
+L<http://rt.perl.org/rt3/Ticket/Display.html?id=60954>
+
+=head2 Calls from XSUBs and Opcodes
+
+Calls record the current filename and line number of the perl code at the time
+the call was made. That's fine and accurate for calls from perl code. For calls
+that originate from C code however, such as an XSUB or an opcode, the filename and
+line number recorded are still those of the last I<perl> statement executed.
+
+For example, a line that calls an xsub will appear in reports to also have also
+called any subroutines that that xsub called. This can be construed as a feature.
+
+As an extreme example, the first time a regular expression that uses character
+classes is executed on a unicode string you'll find profile data like this:
+
+      # spent 1ms within main::BEGIN@4 which was called
+      #    once (1ms+0s) by main::CORE:subst at line 0
+  4   s/ (?: [A-Z] | [\d] )+ (?= [\s] ) //x;
+      # spent  38.8ms making 1 call to main::CORE:subst
+      # spent  25.4ms making 2 calls to utf8::SWASHNEW, avg 12.7ms/call
+      # spent  12.4ms making 1 call to utf8::AUTOLOAD
+
+=for comment
+No doubt more odd cases will be added here over time.
+
 =head1 REPORTS
 
 The L<Devel::NYTProf::Data> module provides a low-level interface for loading
@@ -458,21 +605,80 @@ The L<Devel::NYTProf::Reader> module provides an interface for generating
 arbitrary reports.  This means that you can implement your own output format in
 perl. (Though the module is in a state of flux and may be deprecated soon.)
 
-Included in the bin directory of this distribution are two scripts
-which implement the L<Devel::NYTProf::Reader> interface: 
+There is currently no tool to merge multiple data files into one.
 
-=over 12
+Included in the bin directory of this distribution are some scripts which
+turn the raw profile data into more useful formats:
 
-=item nytprofcsv
+=head2 nytprofcsv
 
-creates comma delimited profile reports
+Creates comma delimited profile reports. Old and limited.
 
-=item nytprofhtml
+=head2 nytprofcg
 
-creates attractive, richly annotated, and fully cross-linked html
-reports (including statistics, source code and color highlighting)
+Translates a profile into a format that can be loaded into KCachegrind
+L<http://kcachegrind.sourceforge.net>
 
-=back
+=head2 nytprofhtml
+
+Creates attractive, richly annotated, and fully cross-linked html
+reports (including statistics, source code and color highlighting).
+
+=head1 LIMITATIONS
+
+=head2 Threads and Multiplicity
+
+C<Devel::NYTProf> is not currently thread safe or multiplicity safe.
+If you'd be interested in helping to fix that then please get in
+touch with us. Meanwhile, profiling is disabled when a thread is created, and
+NYTProf tries to ignore any activity from perl interpreters other than the
+first one that loaded it.
+
+=head2 For perl < 5.8.8 it may change what caller() returns
+
+For example, the L<Readonly> module croaks with "Invalid tie" when profiled with
+perl versions before 5.8.8. That's because L<Readonly> explicitly checking for
+certain values from caller(). The L<NEXT> module is also affected.
+
+=head2 For perl < 5.10.1 it can't see some implicit calls and callbacks
+
+For perl versions prior to 5.8.9 and 5.10.1, some implicit subroutine calls
+can't be seen by the I<subroutine> profiler. Technically this affects calls
+made via the various perl C<call_*()> internal APIs.
+
+For example, BEGIN/CHECK/INIT/END blocks, the C<TIE>I<whatever> subroutine
+called by C<tie()>, all calls made via operator overloading, and callbacks from
+XS code, are not seen.
+
+The effect is that time in those subroutines is accumulated by the
+subs that triggered the call to them. So time spent in calls invoked by
+perl to handle overloading are accumulated by the subroutines that trigger
+overloading (so it is measured, but the cost is dispersed across possibly many
+calling locations).
+
+Although the calls aren't seen by the subroutine profiler, the individual
+I<statements> executed by the code in the called subs are profiled by the
+statement profiler.
+
+=head2 #line directives
+
+The reporting code currently doesn't handle #line directives, but at least it
+warns about them. Patches welcome.
+
+=head2 Freed values in @_ may be mutated
+
+Perl has a class of bugs related to the fact that values placed in the stack
+are not reference counted. Consider this example:
+
+  @a = (1..9);  sub s { undef @a; print $_ for @_ }  s(@a);
+
+The C<undef @a> frees the values that C<@_> refers to. Perl can sometimes
+detect when a freed value is accessed and treats it as an undef. However, if
+the freed value is assigned some new value then @_ is effectively corrupted.
+
+NYTProf allocates new values while it's profiling, in order to record program
+activity, and so may appear to corrupt C<@_> in this (rare) situation.  If this
+happens, NYTProf is simply exposing an existing problem in the code.
 
 =head1 CLOCKS
 
@@ -504,13 +710,13 @@ The problem with real time is that it's far from simple. It tends to drift and
 then be reset to match 'reality', either sharply or by small adjustments (via the
 adjtime() system call).
 
-Surprizingly, it can also go backwards, for reasons explained in
+Surprisingly, it can also go backwards, for reasons explained in
 http://preview.tinyurl.com/5wawnn
 
 =head3 CLOCK_MONOTONIC
 
-CLOCK_MONOTONIC rrepresents the amount of time since an unspecified point in
-the past (typically system start-up time).  It increments uniformally
+CLOCK_MONOTONIC represents the amount of time since an unspecified point in
+the past (typically system start-up time).  It increments uniformly
 independent of adjustments to 'wallclock time'.
 
 =head3 CLOCK_VIRTUAL
@@ -584,49 +790,12 @@ http://groups.google.com/group/comp.os.linux.development.apps/tree/browse_frm/th
 http://webnews.giga.net.tw/article//mailing.freebsd.performance/710
 http://sean.chittenden.org/news/2008/06/01/
 
-=head1 LIMITATIONS
-
-=head2 threads
-
-C<Devel::NYTProf> is not currently thread safe. If you'd be interested in
-helping to make it thread safe then please get in touch with us.
-
-=head2 For perl < 5.8.8 it may change what caller() returns
-
-For example, the L<Readonly> module croaks with "Invalid tie" when profiled with
-perl versions before 5.8.8. That's because L<Readonly> explicitly checking for
-certain values from caller(). The L<NEXT> module is also affected.
-
-=head2 For perl < 5.10.1 it can't see calls made via operator overloading
-
-For perl versions prior to 5.8.9 and 5.10.1, calls made via operator
-overloading are not noticed by the I<subroutine> profiler.  Though the
-statements executed by the code in the overload subs are profiled.
-
-The effect is that time in the subroutines that handle overloading is
-accumulated by any subroutines that trigger overloading (so it is measured,
-but the cost is dispersed across possibly many calling locations).
-
-=head2 goto
-
-The C<goto &foo;> isn't recognised as a subroutine call by the subroutine profiler.
-
-=head2 Calls to XSubs which exit via an exception
-
-Calls to XSubs which exit via an exception are not recorded by subroutine profiler.
-
-=head2 #line directives
-
-The reporting code currently doesn't handle #line directives, but at least it
-warns about them. Patches welcome.
-
 =head1 CAVEATS
 
 =head2 SMP Systems
 
-Systems with multiple processors, which includes most modern machines, have
-
-From Linux docs (though applicable to most SMP systems):
+On systems with multiple processors, which includes most modern machines,
+(from Linux docs though applicable to most SMP systems):
 
   The CLOCK_PROCESS_CPUTIME_ID and CLOCK_THREAD_CPUTIME_ID clocks are realized on
   many platforms using timers from the CPUs (TSC on i386, AR.ITC on Itanium).
@@ -665,9 +834,6 @@ L</SMP Systems>.
 
 Processor affinity can be set using the C<taskset> command on Linux.
 
-Future versions of NYTProf could support setting processor affinity automatically
-(e.g. via sched_setaffinity() on Linux). Patches welcome!
-
 Note that processor affinity is inherited by child processes, so if the process
 you're profiling spawns cpu intensive sub processes then your process will be
 impacted by those more than it otherwise would.
@@ -682,7 +848,7 @@ timings. They will still be noisy but less so than the statement times.
 
 You could also try using the C<clock=N> option to select a high-resolution
 I<cpu-time> clock instead of a real-time one. That should be much less
-noisy, though you will loose visibility of wait-times due to network
+noisy, though you will lose visibility of wait-times due to network
 and disk I/O, for example.
 
 If your system doesn't support the C<clock=N> option then you could try
@@ -728,7 +894,7 @@ B<Steve Peters>, C<< <steve at fisharerojo.org> >>.
 =head1 COPYRIGHT AND LICENSE
 
   Copyright (C) 2008 by Adam Kaplan and The New York Times Company.
-  Copyright (C) 2008 by Tim Bunce, Ireland.
+  Copyright (C) 2008, 2009 by Tim Bunce, Ireland.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
@@ -790,6 +956,6 @@ keep NYTProf working with the latest development perl versions. Nicholas Clark
 added zip compression. Jan Dubois contributed Windows support.
 
 Adam's work is sponsored by The New York Times Co. L<http://open.nytimes.com>.
-Tim's work was partly sponsored by Shopzilla. L<http://www.shopzilla.com>.
+Tim's work was partly sponsored by Shopzilla L<http://www.shopzilla.com> during 2008.
 
 =cut
