@@ -7,11 +7,11 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id: Apache.pm 555 2008-10-27 07:58:56Z gisle.aas $
+# $Id: Apache.pm 998 2009-12-24 14:40:14Z tim.bunce $
 ###########################################################
 package Devel::NYTProf::Apache;
 
-our $VERSION = 0.02;
+our $VERSION = '3.00';
 
 BEGIN {
 
@@ -33,6 +33,8 @@ use constant MP2 => (exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERS
     ? 1
     : 0;
 
+# https://rt.cpan.org/Ticket/Display.html?id=42862
+die "Threads not supported" if $^O eq 'MSWin32';
 
 sub child_init {
     DB::enable_profile() unless $ENV{NYTPROF} =~ m/\b start = (?: no | end ) \b/x;
@@ -117,6 +119,11 @@ command line.
   </IfDefine>
 
 
+=head1 LIMITATIONS
+
+Profiling mod_perl on Windows is not supported because NYTProf currently
+doesn't support threads.
+
 =head1 SEE ALSO
 
 L<Devel::NYTProf>
