@@ -7,7 +7,7 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id: Data.pm 677 2009-01-28 21:57:06Z tim.bunce $
+# $Id: Data.pm 998 2009-12-24 14:40:14Z tim.bunce $
 ###########################################################
 package Devel::NYTProf::Data;
 
@@ -52,13 +52,18 @@ use Devel::NYTProf::FileInfo;
 use Devel::NYTProf::SubInfo;
 use Devel::NYTProf::Util qw(make_path_strip_editor strip_prefix_from_paths get_abs_paths_alternation_regex);
 
-our $VERSION = '2.08';
+our $VERSION = '3.00';
 
 my $trace = (($ENV{NYTPROF}||'') =~ m/\b trace=(\d+) /x) && $1; # XXX a hack
 
 =head2 new
 
-  $profile = Devel::NYTProf::Data->new( { filename => 'nytprof.out' } );
+  $profile = Devel::NYTProf::Data->new( );
+
+  $profile = Devel::NYTProf::Data->new( {
+    filename => 'nytprof.out', # default
+    quiet    => 0,             # default, 1 to silence message
+  } );
 
 Reads the specified file containing profile data written by L<Devel::NYTProf>,
 aggregates the contents, and returns the results as a blessed data structure.
@@ -68,14 +73,16 @@ aggregates the contents, and returns the results as a blessed data structure.
 
 sub new {
     my $class = shift;
-    my $args = shift || { filename => 'nytprof.out' };
+    my $args = shift || { };
 
-    my $file = $args->{filename}
-        or croak "No filename specified";
+    my $file = $args->{filename} ||= 'nytprof.out';
 
     print "Reading $file\n" unless $args->{quiet};
 
-    my $profile = load_profile_data_from_file($file);
+    my $profile = load_profile_data_from_file(
+        $file,
+        $args->{callback},
+    );
     bless $profile => $class;
 
     my $fid_fileinfo = $profile->{fid_fileinfo};
@@ -107,17 +114,19 @@ sub new {
 sub _caches       { return shift->{caches} ||= {} }
 sub _clear_caches { return delete shift->{caches} }
 
+sub attributes {
+    return shift->{attribute} || {};
+}
+
 sub subname_subinfo_map {
     return { %{ shift->{sub_subinfo} } }; # shallow copy
 }
 
-# default:
 # { pkgname => [ subinfo1, subinfo2, ... ], ... }
-# merged:
-# { pkgname => [ single_merged_subinfo ], ...  }
-sub package_subinfo_map {
+# if merged is true then array contains a single 'merged' subinfo
+sub XXXpackage_subinfo_map {
     my $self = shift;
-    my ($merged) = @_;
+    my ($merged_subs, $nested_pkgs) = @_;
 
     my $all_subs = $self->subname_subinfo_map;
     my %pkg;
@@ -125,7 +134,7 @@ sub package_subinfo_map {
         $name =~ s/^(.*::).*/$1/; # XXX $subinfo->package
         push @{ $pkg{$name} }, $subinfo;
     }
-    if ($merged) {
+    if ($merged_subs) {
         while ( my ($pkg_name, $subinfos) = each %pkg ) {
             my $subinfo = shift(@$subinfos)->clone;
             $subinfo->merge_in($_) for @$subinfos;
@@ -136,6 +145,45 @@ sub package_subinfo_map {
     return \%pkg;
 }
 
+# package_tree_subinfo_map is like package_subinfo_map but returns
+# nested data instead of flattened.
+# for "Foo::Bar::Baz" package:
+# { Foo => { '' => [...], '::Bar' => { ''=>[...], '::Baz'=>[...] } } }
+# if merged is true then array contains a single 'merged' subinfo
+sub package_subinfo_map {
+    my $self = shift;
+    my ($merge_subs, $nested_pkgs) = @_;
+
+    my %pkg;
+    my %to_merge;
+
+    my $all_subs = $self->subname_subinfo_map;
+    while ( my ($name, $subinfo) = each %$all_subs ) {
+        $name =~ s/^(.*::).*/$1/; # XXX $subinfo->package
+        my $subinfos;
+        if ($nested_pkgs) {
+            my @parts = split /::/, $name;
+            my $node = $pkg{ shift @parts } ||= {};
+            $node = $node->{ shift @parts } ||= {} while @parts;
+            $subinfos = $node->{''} ||= [];
+        }
+        else {
+            $subinfos = $pkg{$name} ||= [];
+        }
+        push @$subinfos, $subinfo;
+        $to_merge{$subinfos} = $subinfos if $merge_subs;
+    }
+
+    for my $subinfos (values %to_merge) {
+        my $subinfo = shift(@$subinfos)->clone;
+        $subinfo->merge_in($_) for @$subinfos;
+        # replace the many with the one
+        @$subinfos = ($subinfo);
+    }
+
+    return \%pkg;
+}
+
 # [
 #   undef,  # depth 0
 #   {       # depth 1
@@ -284,35 +332,37 @@ sub dump_profile_data {
 
     $self->_clear_caches;
 
-    my $abs_path_regex = $^O eq "MSWin32" ? qr,^\w:/, : qr,^/,;
-    my @abs_inc = grep { $_ =~ $abs_path_regex } $self->inc;
-    my $is_lib_regex = get_abs_paths_alternation_regex(\@abs_inc);
-
     my $callback = sub {
         my ($path, $value) = @_;
 
+        if ($path->[0] eq 'attribute' && @$path == 1) {
+            my %v = %$value;
+            delete @v{qw(PRIVLIB_EXP ARCHLIB_EXP)};
+            return ({}, \%v);
+        }
+
         if ($args->{skip_stdlib}) {
 
             # for fid_fileinfo don't dump internal details of lib modules
             if ($path->[0] eq 'fid_fileinfo' && @$path==2) {
-                my $is_lib = ($value->filename =~ $is_lib_regex) ? 1 : 0;
-                return { skip_internal_details => $is_lib };
+                my $fi = $self->fileinfo_of($value->[0]);
+                return ({ skip_internal_details => $fi->is_perl_std_lib }, $value);
             }
 
             # skip sub_subinfo data for 'library modules'
             if ($path->[0] eq 'sub_subinfo' && @$path==2 && $value->[0]) {
                 my $fi = $self->fileinfo_of($value->[0]);
-                return undef if $fi->filename =~ $is_lib_regex;
+                return undef if $fi->is_perl_std_lib;
             }
 
             # skip fid_*_time data for 'library modules'
             if ($path->[0] =~ /^fid_\w+_time$/ && @$path==2) {
                 my $fi = $self->fileinfo_of($path->[1]);
-                return undef if $fi->filename =~ $is_lib_regex
+                return undef if $fi->is_perl_std_lib
                          or $fi->filename =~ m!^/\.\.\./!;
             }
         }
-        return {};
+        return ({}, $value);
     };
 
     _dump_elements($startnode, $separator, $filehandle, [], $callback);
@@ -352,7 +402,7 @@ sub _dump_elements {
 
         my $dump_opts = {};
         if ($callback) {
-            $dump_opts = $callback->([ @$path, $key ], $value);
+            ($dump_opts, $value) = $callback->([ @$path, $key ], $value);
             next if not $dump_opts;
         }
 
@@ -441,6 +491,7 @@ filenames: eval sequence numbers, like "(re_eval 2)" are changed to 0
 
 sub normalize_variables {
     my $self       = shift;
+    my $attributes = $self->attributes;
 
     for my $attr (qw(
         basetime xs_version perl_version clock_id ticks_per_sec nv_size
@@ -448,13 +499,12 @@ sub normalize_variables {
         total_stmts_duration total_stmts_measured total_stmts_discounted
         total_sub_calls
     )) {
-        $self->{attribute}{$attr} = 0;
+        $attributes->{$attr} = 0;
     }
 
-    my $abs_path_regex = $^O eq "MSWin32" ? qr,^\w:/, : qr,^/,;
-    my @abs_inc = grep { $_ =~ $abs_path_regex } $self->inc;
-    my $is_lib_regex = get_abs_paths_alternation_regex(\@abs_inc);
-
+    for my $attr (qw(PL_perldb)) {
+        delete $attributes->{$attr};
+    }
 
     # normalize line data
     for my $level (qw(line block sub)) {
@@ -557,6 +607,8 @@ sub subs_defined_in_file {
   @subname = $profile->subname_at_file_line($file, $line_number);
   $subname = $profile->subname_at_file_line($file, $line_number);
 
+This method is currently unused and may be deprecated.
+
 =cut
 
 
@@ -634,21 +686,25 @@ sub file_line_range_of_sub {
         or return;    # no such sub
     my ($fid, $first, $last) = @$sub_subinfo;
 
-    my $fileinfo = $fid && $self->{fid_fileinfo}->[$fid];
-    while ($fileinfo->[1]) {    # file is an eval
+    return if not $fid; # sub has no known file
+
+    my $fileinfo = $fid && $self->{fid_fileinfo}->[$fid]
+        or die "No fid_fileinfo for sub $sub fid '$fid'\n";
+    while ($fileinfo->eval_fid) {
 
         # eg string eval
         # eg [ "(eval 6)[/usr/local/perl58-i/lib/5.8.6/Benchmark.pm:634]", 2, 634 ]
-        warn sprintf "file_line_range_of_sub: %s: fid %d -> %d for %s\n", $sub, $fid, $fileinfo->[1], $fileinfo->[0]
+        warn sprintf "file_line_range_of_sub: %s: fid %d -> %d for %s\n",
+                $sub, $fid, $fileinfo->eval_fid, $fileinfo->filename
             if $trace;
-        $first = $last = $fileinfo->[2] if 1;    # XXX control via param?
+        $first = $last = $fileinfo->eval_line if 1;    # XXX control via param?
 
         # follow next link in chain
-        my $outer_fid = $fileinfo->[1];
+        my $outer_fid = $fileinfo->eval_fid;
         $fileinfo = $self->{fid_fileinfo}->[$outer_fid];
     }
 
-    return ($fileinfo->[0], $fid, $first, $last);
+    return ($fileinfo->filename, $fid, $first, $last);
 }
 
 
@@ -798,6 +854,10 @@ __END__
 
 XXX
 
+=head1 LIMITATION
+
+There's currently no way to merge profile data from multiple files.
+
 =head1 SEE ALSO
 
 L<Devel::NYTProf>
@@ -811,7 +871,7 @@ B<Steve Peters>, C<< <steve at fisharerojo.org> >>
 =head1 COPYRIGHT AND LICENSE
 
  Copyright (C) 2008 by Adam Kaplan and The New York Times Company.
- Copyright (C) 2008 by Tim Bunce, Ireland.
+ Copyright (C) 2008,2009 by Tim Bunce, Ireland.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
