@@ -3,21 +3,21 @@ package Devel::NYTProf::ReadStream;
 use warnings;
 use strict;
 
-our $VERSION = '2.08';
+our $VERSION = '3.00';
 
 use base 'Exporter';
 our @EXPORT_OK = qw(
      for_chunks
 );
 
-use Devel::NYTProf::Core;
+use Devel::NYTProf::Data;
 
 sub for_chunks (&%) {
     my($cb, %opts) = @_;
-    Devel::NYTProf::Data::load_profile_data_from_file(
-	$opts{filename} || 'nytprof.out',
-	$cb,
-    );
+    Devel::NYTProf::Data->new( {
+        %opts,
+	callback => $cb,
+    });
 }
 
 1;
@@ -67,19 +67,12 @@ to track the chunk sequence numbers and can be inspected in the
 callback.
 
 The behaviour of the function can be modified by passing key/value
-pairs after the callback.  Currently recognized are:
-
-=over
-
-=item filename => $path
-
-The path to the data file to read.  Defaults to F<nytprof.out>.
-
-=back
+pairs after the callback. The contents of %opts are passed to
+L<Devel::NYTProf::Data/new>.
 
 The function is prototyped as C<(&%)> which means that it can be invoked with a
-bare block representing the callback function.  In that case there
-should be no comma before any options.  Example:
+bare block representing the callback function.  In that case there should be no
+comma before any options.  Example:
 
   for_chunk { say $_[0] } filename => "myprof.out";
 
@@ -184,13 +177,13 @@ the program leaves a statement.
 Indicates that the next TIME_BLOCK or TIME_LINE should not increment the
 "number of times the statement was executed". See the 'leave' option.
 
-=item SUB_LINE_RANGE => $fid, $beg, $end, $name
+=item SUB_INFO => $fid, $first_line, $last_line, $name
 
 At the end of the run the profiler will output chunks that report on
 the perl subroutines defined in all the files visited while profiling.
-This is a straight dump C<%DB::sub>; see L<perldebguts>.
+See also C<%DB::sub> in L<perldebguts>.
 
-=item SUB_CALLERS => $fid, $line, $count, $incl_time, $excl_time, $ucpu_time, $scpu_time, $reci_time, $rec_depth, $name
+=item SUB_CALLERS => $fid, $line, $count, $incl_time, $excl_time, $ucpu_time, $scpu_time, $reci_time, $rec_depth, $name, $caller_name
 
 At the end of the run the profiler will output chunks that report on
 where subroutines were called from.
