@@ -7,7 +7,7 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id: Util.pm 646 2008-12-09 16:00:18Z tim.bunce $
+# $Id: Util.pm 998 2009-12-24 14:40:14Z tim.bunce $
 ###########################################################
 package Devel::NYTProf::Util;
 
@@ -38,9 +38,9 @@ use base qw'Exporter';
 use Carp;
 use Cwd qw(getcwd);
 use List::Util qw(sum);
-use UNIVERSAL qw( isa can VERSION );
+#use UNIVERSAL qw( isa can VERSION );
 
-our $VERSION = '2.08';
+our $VERSION = '3.00';
 
 our @EXPORT_OK = qw(
     fmt_float
@@ -162,11 +162,15 @@ sub fmt_float {
 }
 
 
+# XXX undocumented hack that may become to an option one day
+my $fmt_time_opt = $ENV{NYTPROF_FMT_TIME}; # e.g., '%fs'
+
 sub fmt_time {
     my ($sec, $width) = @_;
     $width = '' unless defined $width;
     return sprintf "%$width.0fs", 0    unless $sec;
     return '-'.fmt_time(-$sec, $width) if $sec < 0; # negative value, can happen
+    return sprintf $fmt_time_opt, $sec if $fmt_time_opt;
     return sprintf "%$width.0fns",                              $sec * 1e9 if $sec < 1e-6;
     return sprintf "%$width.0f&micro;s",                        $sec * 1e6 if $sec < 1e-3;
     return sprintf "%$width.*fms", 3 - length(int($sec * 1e3)), $sec * 1e3 if $sec < 1;
@@ -216,8 +220,19 @@ sub calculate_median_absolute_deviation {
 
 sub html_safe_filename {
     my ($fname) = @_;
-    $fname =~ s{ ^[/\\] }{}x;                      # remove leading / or \
-    $fname =~ s{  [/\\] }{-}xg;                    # replace / and \ with html safe -
+    # replace / and \ with html safe '-', we also do a bunch of other
+    # chars, especially ':' for Windows, to make the namer simpler and safer
+    # also remove dots to keep VMS happy
+    $fname =~ s{  [-/\\:\*\?"'<>|.]+ }{-}xg;
+    # remove any leading or trailing '-' chars
+    $fname =~ s{^-}{};
+    $fname =~ s{-$}{};
+    if($^O eq 'VMS'){
+        # ODS-2 is limited to 39.39 chars (39 filename, 39 extension)
+	# Reader.pm appends -LEVEL onto html safe filename so must
+	# subtract 1 + max length of (sub block line), so 6.
+        $fname = substr($fname,-33);
+    }
     return $fname;
 }
 
